// @dart = 2.8

import 'dart:async';

import 'package:yaml/yaml.dart';

import '../base/common.dart';
import '../globals.dart';
import '../runner/faircli_command.dart';
import '../util/util.dart';

const fairVersion = '^2.4.1';
const fairCompiler = '^1.1.0';
const buildRunnerVersion = '^2.0.0';

class ImportCommand extends FaircliCommand {
  ImportCommand() {
    argParser.addOption(
      'description',
      defaultsTo: 'Import Fair dependency to your carrier project.',
      help: 'The description to use for your Fair project.',
    );

    argParser.addOption(
      'directory',
      abbr: 'd',
      defaultsTo: '',
      help: 'flutter project directory',
    );
  }

  @override
  final String name = 'import';

  @override
  final String description = 'Import Fair dependency to your carrier project.';

  @override
  Future<FaircliCommandResult> runCommand() async {
    String directory = argResults['directory'];
    if (directory.isEmpty) {
      throwToolExit(
          'Please use "faircli import -d xxx" to import Fair dependency to your carrier project');
    }
    var projectDir = fs.directory(directory);
    if (!projectDir.existsSync()) {
      throwToolExit('The input directory is not exists');
    }
    cliPrint('Start to import Fair dependency, wait a moment please...');
    // add Fair dependency
    var pubspecFile = fs.file(fs.path.join(directory, 'pubspec.yaml'));
    if (!pubspecFile.existsSync()) {
      throwToolExit(
          'The directory is not a Flutter project, "pubspec.yaml" is not exists');
    }
    YamlMap yamlData;
    try {
      yamlData = loadYaml(pubspecFile.readAsStringSync());
    } catch (e) {
      throwToolExit('loadYaml failed: $e');
    }
    var hasDevDependencies = false;
    var hasDependencyOverrides = false;
    var pubspecStr = StringBuffer();
    pubspecFile.readAsLinesSync().forEach((line) {
      if (line.trim() == 'dependencies:') {
        pubspecStr.writeln(line);
        _writeFairDependency(yamlData, pubspecStr);
      } else if (line.trim() == 'dev_dependencies:') {
        pubspecStr.writeln(line);
        hasDevDependencies = true;
        _writeFairDevDependency(yamlData, pubspecStr);
      } else if (line.trim() == 'dependency_overrides:') {
        pubspecStr.writeln(line);
        hasDependencyOverrides = true;
        _writeFairDevDependencyOverrides(yamlData, pubspecStr);
      } else {
        pubspecStr.writeln(line);
      }
    });
    if (!hasDevDependencies) {
      pubspecStr.writeln('dev_dependencies:');
      _writeFairDevDependency(yamlData, pubspecStr);
    }
    if (!hasDependencyOverrides) {
      pubspecStr.writeln('dependency_overrides:');
      _writeFairDevDependencyOverrides(yamlData, pubspecStr);
    }
    pubspecFile.writeAsStringSync(pubspecStr.toString());
    cliPrint('Import Fair dependency success');
    cliPrint('''
Then you should modify the 'main.dart' like this:

void main() {
  WidgetsFlutterBinding.ensureInitialized();

  FairApp.runApplication(
    _getApp(),
    plugins: {
    },
  );
}

dynamic _getApp() => FairApp(
  modules: {
  },
  delegate: {
  },
  child: MaterialApp(
    home: FairWidget(
            name: 'DynamicWidget',
            path: 'assets/bundle/lib_src_page_dynamic_widget.fair.json',
            data: {"fairProps": json.encode({})}),
  ),
);

Enjoy~
    ''');
    return null;
  }

  void _writeFairDependency(YamlMap yamlData, StringBuffer pubspecStr) {
    if (!_yamlExistsDependency(yamlData, 'dependencies', 'fair')) {
      pubspecStr.writeln('  fair: $fairVersion');
    }
  }

  void _writeFairDevDependencyOverrides(
      YamlMap yamlData, StringBuffer pubspecStr) {
    pubspecStr.writeln('#  fair_version:');
    pubspecStr.writeln('#    path: ../flutter_version/flutter_2_10_0');
  }

  void _writeFairDevDependency(YamlMap yamlData, StringBuffer pubspecStr) {
    if (!_yamlExistsDependency(yamlData, 'dev_dependencies', 'fair_compiler')) {
      pubspecStr.writeln('  fair_compiler: $fairCompiler');
    }
    if (!_yamlExistsDependency(yamlData, 'dev_dependencies', 'build_runner')) {
      pubspecStr.writeln('  build_runner: $buildRunnerVersion');
    }
  }

  bool _yamlExistsDependency(YamlMap yamlData, String cate, String package) {
    if (!yamlData.containsKey(cate)) {
      return false;
    }
    return yamlData[cate][package] != null;
  }
}
